local BG = LibStub("AceAddon-3.0"):GetAddon("Battleground")
local module = BG:NewModule("IsleofConquest", "AceEvent-3.0")

local L = LibStub("AceLocale-3.0"):GetLocale("Battleground")

local landmarks = BG.landmarks
local namedButtons = BG.namedButtons

module.captureTimer = 62
module.startTimer = 70

local pName = UnitName("player")

module.buttons = {
	left = {
		L["Horde Keep"],
		L["Refinery"],
		L["Docks"],
	},
	right = {
		L["Alliance Keep"],
		L["Hangar"],
		L["Quarry"],
	},
	bottom = {
		L["Workshop"],
		L["Siege Engine"],
	},
	top = {
		L["Battleground Start"],
	},
}

-- damn exceptions in chat
local pois = {
	[L["horde keep"]] = L["Horde Keep"],
	[L["alliance keep"]] = L["Alliance Keep"],
	[L["oil refinery"]] = L["Refinery"],
	[L["docks"]] = L["Docks"],
	[L["hangar"]] = L["Hangar"],
	[L["quarry"]] = L["Quarry"],
	[L["workshop"]] = L["Workshop"],
}


-- since we're syncing with capping anyway, I use its id's
local sieges = {
	[L["Gnomish Mechanic"]] = {
		[L["See those seaforium bombs? Use them on the gates while I repair the siege engine!"]] = "a1",
		[L["I'm halfway there! Keep the Horde away from here.  They don't teach fighting in engineering school!"]] = "a2",
		[L["My finest work so far! This siege engine is ready for action!"]] = FACTION_ALLIANCE
	},
	[L["Goblin Mechanic"]] = {
		[L["I'll work on the siege engine, just watch my back.  Use those seaforium bombs on the gates if you need them!"]] = "h1",
		[L["I'm about halfway done! Keep the Alliance away - fighting's not in my contract!"]] = "h2",
		[L["The siege engine is ready to roll!"]] = FACTION_HORDE,
	}
}

local chatFilters = {
	[L["High Commander Halford Wyrmbane"]] = FACTION_HORDE,
	[L["Overlord Agmar"]] = FACTION_ALLIANCE
}
-- special siege onupdate
local function OnUpdate(self, elapsed)
	if not self.expire then return end
	if self.expire >= time() then
		local tleft = self.expire - time()
		local m = floor(tleft/60)
		local s = tleft % 60
		self.time:SetFormattedText("%0d:%02d", m, s)
	else
		self:SetScript("OnUpdate", nil)
		self:SetAlpha(.6)
		self.time:SetText(nil)
		self.text:SetTextColor(.6,.6,.6)
		if self.status then
			module:SendMessage("Battleground_Message", L["The %s has finished a siege engine."]:format(self.status), self.status, chatFrame or DEFAULT_CHAT_FRAME)
		end
	end
end
local function handleSiege(id, chatFrame)
	local b = namedButtons[L["Siege Engine"]]
	if siege == FACTION_HORDE or siege == FACTION_ALLIANCE then
		b.expire = time()
	else
		local faction, remain = strmatch(id, "(%a)(%d+)")
		faction = (faction == "a" and FACTION_ALLIANCE) or (faction == "h" and FACTION_HORDE)
		if remain == "1" then
			module:SendMessage("Battleground_Message", L["The %s has started building a siege engine."]:format(faction), faction, chatFrame or DEFAULT_CHAT_FRAME)
		elseif remain == "2" then
			module:SendMessage("Battleground_Message", L["The %s is halfway building a siege engine."]:format(faction), faction, chatFrame or DEFAULT_CHAT_FRAME)
		end
		remain = (remain == "1" and 183) or (remain == "2" and 92) or tonumber(remain or 0) or 0
		if remain > 0 then
			b.expire = time() + remain
			local color = faction == FACTION_ALLIANCE and ChatTypeInfo["BG_SYSTEM_ALLIANCE"] or ChatTypeInfo["BG_SYSTEM_HORDE"]
			b.status = faction
			b.text:SetTextColor(color.r, color.g, color.b)
			b.time:SetTextColor(color.r, color.g, color.b)
			b:SetAlpha(1)
			b:SetScript("OnUpdate", OnUpdate)
		end
	end
end

local function filterFunc(chatFrame, event, msg, author, ...)
	if not BattlegroundDB then BattlegroundDB = {} end
	if not BattlegroundDB.ioclog then BattlegroundDB.ioclog = {} end
	if not BattlegroundDB.ioclog[author] then BattlegroundDB.ioclog[author] = {} end
	BattlegroundDB.ioclog[author][msg] = true
	
	if sieges[author] and sieges[author][msg] then
		handleSiege(sieges[author][msg])
		SendAddonMessage("cap", sieges[author][msg], "BATTLEGROUND") -- be nice to capping users
		return true
	elseif chatFilters[author] and msg == L["The gates have been breached! Defend the keep!"] then
		module:SendMessage("Battleground_Message", L["The %s has breached the %s Keep!"]:format(chatFilters[author], chatFilters[author] == FACTION_HORDE and FACTION_ALLIANCE or FACTION_HORDE), chatFilters[author], chatFrame)
		return true
	end
end

function module:OnEnable()
	ChatFrame_AddMessageEventFilter("CHAT_MSG_MONSTER_YELL", filterFunc)
	self:RegisterEvent("CHAT_MSG_ADDON")
	self:RegisterEvent("CHAT_MSG_BG_SYSTEM_HORDE", "ParsePerson")
	self:RegisterEvent("CHAT_MSG_BG_SYSTEM_ALLIANCE", "ParsePerson")
	self:RegisterMessage("Battleground_POIUpdate")
	
	local b = namedButtons[L["Siege Engine"]]
	local _, _, icon = GetSpellInfo(67323)
	b.icon:SetTexture( icon )
	b.icon:SetTexCoord(0.03, 0.97, 0.03, 0.97)
	b.text:SetTextColor(.6,.6,.6)
	b:SetAlpha(.6)
end

function module:ParsePerson(event, msg)
	local name, poi = msg:match(L["(.+) claims the ([^!]+)"])
	if not name then
		name, poi = msg:match(L["(.+) has assaulted the ([^!]+)"])
	end
	if name then
		name = name:trim()
		poi = poi:trim()
		poi = pois[poi:lower()]
		for k, v in pairs(namedButtons) do
			if k == poi then
				if v.align == "left" then
					v.text:SetFormattedText("[%s] %s", name, k)
				else
					v.text:SetFormattedText("%s [%s]", k, name)
				end
				break
			end
		end
		return
	end
	poi = msg:match(L["has taken the ([^!]+)"])
	if not poi then
		poi = msg:match(L["has defended the ([^!]+)"])
	end
	if poi then
		poi = poi:trim()
		poi = pois[poi:lower()]
		for k, v in pairs(namedButtons) do
			if k == poi then
				v.text:SetText(k)
				break
			end
		end
		return
	end
end

function module:Battleground_POIUpdate(message, name, desc, ti, button, x, y)
	if name == L["Workshop"] and ( landmarks[ti][2] == "horde_assaulted" or landmarks[ti][2] == "alliance_assaulted" ) then
		local b = namedButtons[L["Siege Engine"]]
		b.status = nil
		b.expire = time() -- cancel siege timer
	end
end

function module:CHAT_MSG_ADDON(event, prefix, message, chan, sender)
	if ( prefix == "cap" or prefix == "Cap" ) and sender ~= pName then
		handleSiege(message)
	end
end
